// GSMUSTOR.INL
//
// Copyright (c) 1999-2001 Symbian Ltd.  All rights reserved.
//

/**
 * @file gsmustor.inl Implements inline methods for classes TSAREntry and CSARStore
 *
 */

#include <s32file.h>

inline TInt TSAREntry::Reference() const
/** Gets the Reference field.

@return Concatenation reference */
	{
	return iReference;
	}


inline void TSAREntry::SetReference(TInt aReference)
/** Sets the Reference field.

@param aReference The concatenation reference */
	{
	iReference=aReference;
	}

inline TInt TSAREntry::Total() const
/** Gets the Total field.

@return The total number of PDUs in SMS message */
	{
	return iTotal;
	}

inline void TSAREntry::SetTotal(TInt aTotal)
/** Sets the total number of PDUs in SMS message
 
@param aTotal The total number of PDUs in the message */
	{
	iTotal=aTotal;
	}

inline TInt TSAREntry::Count() const
/** Gets the Count field.

@return Number of PDUs sent/received */
	{
	return iCount;
	}

inline void TSAREntry::SetCount(TInt aCount)
/** Sets the Count field.

@param aCount The number of PDUs sent/received */
	{
	iCount=aCount;
	}

inline TBool TSAREntry::IsComplete() const
/** Tests if the whole message is sent/received.

@return True if the whole message is sent/received */
	{
	return iCount==iTotal;
	}

inline TPtrC TSAREntry::Description1() const
/** Gets the Description1 field: a pointer to the first 
32 characters from buffer.

@return Description1 field */
	{
	return iDescription1;
	}

inline void TSAREntry::SetDescription1(const TDesC& aDescription)
/** Sets the Description1 field: the first 32 characters in buffer.

@param aDescription Description1 field */
	{
	iDescription1=aDescription;
	}

inline TPtrC TSAREntry::Description2() const
/** Gets the Description2 field: a pointer to the original address.

@return Description2 field */
	{
	return iDescription2;
	}

inline void TSAREntry::SetDescription2(const TDesC& aDescription)
/** Sets the Description2 field: the original address.

@param aDescription Description2 field */
	{
	iDescription2=aDescription;
	}

inline TInt TSAREntry::Data3() const
/** Gets the Data3 field.

@return Data3 field */
	{
	return iData3;
	}

inline void TSAREntry::SetData3(TInt aData)
/** Sets the Data3 field.

@param aData Data3 field */
	{
	iData3=aData;
	}

inline const TTime& TSAREntry::Time() const
/** Gets the Time field: the time when this message was sent/received.

@return Time field */
	{
	return iTime;
	}

inline void TSAREntry::SetTime(const TTime& aTime)
/** Sets the Time field: the time when this message was sent/received.

@param aTime Time field */
	{
	iTime=aTime;
	}

inline TStreamId TSAREntry::DataStreamId() const
/** Gets the data stream ID.

@return Data stream ID */
	{
	return iDataStreamId;
	}

inline void TSAREntry::SetDataStreamId(TStreamId aStreamId)
/** Sets the data stream ID.

@param aStreamId Data stream ID */
	{
	iDataStreamId=aStreamId;
	}

inline TBool TSAREntry::IsDeleted() const
/**
@return ETrue if this message is deleted
*/
	{
	return iFlags&ESAREntryIsDeleted;
	}

inline void TSAREntry::SetIsDeleted(TBool aIsDeleted)
/**
Sets if this message should be marked as deleted
*
@param aIsDeleted Set to True to mark a message as deleted
*/
	{
	iFlags=aIsDeleted?iFlags|ESAREntryIsDeleted: iFlags&(~ESAREntryIsDeleted);
	}

inline TBool TSAREntry::IsAdded() const
/**
@return ETrue if this message is added
*/
	{
	return iFlags&ESAREntryIsAdded;
	}

inline void TSAREntry::SetIsAdded(TBool aIsAdded)
/**
Sets if this message should be marked as added

@param aIsDeleted Set to True to mark a message as added
*/
	{
	iFlags=aIsAdded?iFlags|ESAREntryIsAdded: iFlags&(~ESAREntryIsAdded);
	}

inline RFs& CSARStore::FileSession() const
/** Gets the file server handle.

@return File server handle */
	{
	return iFs;
	}

inline TBool CSARStore::InTransaction() const
/**
@return whether it's in transaction
*/
	{
	return iInTransaction;
	}
